% sigma(1) is the sigma within each row, sigma(2) is within each column.
% If sigma is a scalar, sigma(1) == sigma(2).
function s = smooth(img, sigma, pad)

if nargin < 2
    sigma = 1;
end

if nargin < 3
    pad = 0;
end

if ndims(img) == 3   % Do one band at a time
    s = [];
    for b = 1:ndims(img)
        band = squeeze(img(:, :, b));
        s = cat(3, s, smooth(band, sigma, pad));
    end
elseif ndims(img) == 2
	if length(sigma) == 1
        sigma = sigma * [1 1];
	end
	tail = ceil(3.5 * sigma);
	r = -tail(1):tail(1);
	r = r(:);	
	gr = gauss(r, 0, sigma(1), 1);
	c = -tail(2):tail(2);
	c = c(:);	
	gc = gauss(c, 0, sigma(2), 1);
	
	img = double(img);
	
	if pad  % Pad by pixel replication around boundaries
        [rows, cols] = size(img);
        bigimg = zeros(rows + 2 * tail(2), cols + 2 * tail(1));
        rs = tail(2) + 1;
        re = rs + rows - 1;
        cs = tail(1) + 1;
        ce = cs + cols - 1;
        bigimg(rs:re, cs:ce) = img;
        bigimg(rs:re, 1:tail(1)) = img(:, 1) * ones(1, tail(1));
        bigimg(rs:re, (end-tail(1)+1):end) = img(:, end) * ones(1, tail(1));
        bigimg(1:tail(2), :) = ones(tail(2), 1) * bigimg(tail(2)+1, :);
        bigimg((end-tail(2)+1):end, :) = ones(tail(2), 1) * bigimg(end-tail(2), :);
        s = conv2(gc, gr, bigimg, 'valid');
	else
        s = conv2(gc, gr, img, 'same');
	end
else
    throwError('smooth:BadSize', 'Images must be two or three-dimensional')
end